@@ -38,10 +38,12 @@ def unpack_phase(phase, svid="") -> tuple:
         hp_i, ps = phase
     except TypeError:
         if phase is None:
-            hp_i, ps = ".", "."
+            hp_i, ps = None, "."
         else:
             log.debug(f"Single not 'None'-valued phase: {phase}|{svid}")
             hp_i, ps = phase, phase
+    if hp_i is None or ps is None:
+        return None, "."
     ps = ps if ps is not None else "."
     return hp_i, ps
 
@@ -57,7 +59,7 @@ def format_genotype(gt, is_phased):
     if len(gt) == gt_multi_sample_fields:
         a, b, qual, dr, dv, phase = gt
         hp_i, ps = unpack_phase(phase)
-        if hp_i is not None and (a, b) == (0, 1) and is_phased:
+        if hp_i is not None and (a, b) in [(0, 1), (1, 1)] and is_phased:
             gt_sep = "|"
             if hp_i == 0:
                 a, b = b, a
@@ -67,7 +69,7 @@ def format_genotype(gt, is_phased):
     else:
         a, b, qual, dr, dv, phase, svid = gt
         hp_i, ps = unpack_phase(phase, svid)
-        if hp_i is not None and (a, b) == (0, 1) and is_phased:
+        if hp_i is not None and (a, b) == [(0, 1), (1, 1)]  and is_phased:
             gt_sep = "|"
             if hp_i == 0:
                 a, b = b, a
@@ -160,6 +162,7 @@ class VCF:
         self.write_header_line('FILTER=<ID=STRAND_MOSAIC,Description="Strand support filter for mosaic SVs">')
         self.write_header_line('FILTER=<ID=SVLEN_MIN,Description="SV length filter">')
         self.write_header_line('FILTER=<ID=SVLEN_MIN_MOSAIC,Description="SV length filter for mosaic SVs">')
+        self.write_header_line('FILTER=<ID=SINGLE_BREAK,Description="A single break point was detected but not classified as an SV.">')
         self.write_header_line('INFO=<ID=PRECISE,Number=0,Type=Flag,Description="Structural variation with precise breakpoints">')
         self.write_header_line('INFO=<ID=IMPRECISE,Number=0,Type=Flag,Description="Structural variation with imprecise breakpoints">')
         self.write_header_line('INFO=<ID=MOSAIC,Number=0,Type=Flag,Description="Structural variation classified as putative mosaic">')
@@ -194,12 +197,8 @@ class VCF:
         self.write_raw(f"#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\tFORMAT\t{samples_header}")
 
     def write_raw(self, text, endl="\n"):
-        if self.config.vcf_output_bgz:
-            self.handle.write(text.encode())
-            self.handle.write(endl.encode())
-        else:
-            self.handle.write(text)
-            self.handle.write(endl)
+        self.handle.write(text)
+        self.handle.write(endl)
 
     def write_header_line(self, text):
         self.write_raw("##" + text)
@@ -211,6 +210,9 @@ class VCF:
         end = call.end
         pos = call.pos if call.pos > 0 else 1
 
+        if call.precise and call.svtype == 'DEL':
+            end = pos + abs(call.svlen)
+
         # Determine genotypes columns
         ac = 0  # Allele count
         supvec = []
@@ -267,10 +269,6 @@ class VCF:
             infos["SVLENGTHS"] = None
             infos["END"] = None
 
-        if call.svtype == "DEL":
-            # END is POS + length of REF allele - 1
-            infos["END"] = end - 1
-
         infos_ordered = ["PRECISE" if call.precise else "IMPRECISE"]
         af = call.get_info("VAF")
         af = af if af is not None else 0
